#%%
import subprocess
import os
import time
import glob

# 1. Create a list of file names
file_names = ["beam-1elem", "beam-5elem", "beam-1elem-quad", "beam-2elem-quad", "beam-4elem-quad"]

# Dictionary to map file names to elements and node counts
file_metadata = {
    file_names[0]: ("u2-beam", "1/2"),
    file_names[1]: ("u2-beam", "5/6"),
    file_names[2]: ("u3-beam", "1/3"),
    file_names[3]: ("u3-beam", "2/5"),
    file_names[4]: ("u3-beam", "4/9")
}

# analytical solution value
analytical_solution = 6.76

results = []

# 2. Process each file
for filename in file_names:
    print(f"Processing {filename}...")
    
    cmd = f"numgeo inp={filename} ncpus=1"
    print(f"Running: {cmd}")
    try:
        process = subprocess.run(cmd, shell=True, text=True, capture_output=True)
        if process.returncode != 0:
            print(f"Error running command: {process.stderr}")
            continue
    except Exception as e:
        print(f"Exception occurred: {str(e)}")
        continue
    
    output_dir = f"./{filename}-print-out/"
    output_file = os.path.join(output_dir, 'LOAD*.dat')

    time.sleep(1)

    try:
        
        # Find all files matching the pattern LOAD*.dat in the output directory
        load_files = glob.glob(os.path.join(output_dir, "LOAD*.dat"))
        
        if not load_files:
            print(f"No LOAD*.dat files found in {output_dir}")
            continue
            
        # Use the first matching file (or you could process all of them)
        output_file = load_files[0]
        print(f"Reading data from {output_file}")
            
        with open(output_file, 'r') as f:
            lines = f.readlines()
            
        if not lines:
            print(f"File is empty: {output_file}")
            continue
            
        # C. Extract the last value from the 4th column
        last_line = lines[-1].strip()
        columns = last_line.split()
        
        if len(columns) < 3:
            print(f"Last line doesn't have enough columns: {last_line}")
            continue
            
        last_value = -float(columns[2]) * 1000.
        
        # 3. Calculate percentage difference
        diff = last_value - analytical_solution
        percent_diff = (diff / analytical_solution) * 100
        
        # Get metadata for this file
        element_type, elements_nodes = file_metadata.get(filename, ("Unknown", "Unknown"))
        
        # Store result
        results.append({
            "Element": element_type,
            "Number of elements/nodes": elements_nodes,
            "Max. deflection": f"{last_value:.3f} mm",
            "Difference": f"{'+' if percent_diff >= 0 else ''}{percent_diff:.1f} %"
        })
        
    except Exception as e:
        print(f"Error processing {filename}: {str(e)}")

# 4. Print results as a markdown table
print("\n|Element|Number of <br>elements/nodes|Max. deflection|Difference to <br>analytical solution|")
print("|-------|-------|-------|-------|")
for result in results:
    print(f"|{result['Element']}|{result['Number of elements/nodes']}|{result['Max. deflection']}|{result['Difference']}|")